@@ -288,10 +288,14 @@ void xio_session_read_header(struct xio_task *task,
 /*---------------------------------------------------------------------------*/
 void xio_session_notify_teardown(struct xio_session *session, int reason)
 {
-	struct xio_session_event_data  event = {};
-
-	event.event = XIO_SESSION_TEARDOWN_EVENT;
-	event.reason = (enum xio_status)reason;
+	struct xio_session_event_data  event = {
+		.conn = NULL,
+		.conn_user_context = NULL,
+		.event = XIO_SESSION_TEARDOWN_EVENT,
+		.reason = (enum xio_status)reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -312,12 +316,14 @@ void xio_session_notify_teardown(struct xio_session *session, int reason)
 void xio_session_notify_new_connection(struct xio_session *session,
 				       struct xio_connection *connection)
 {
-	struct xio_session_event_data  event = {};
-
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
-	event.event = XIO_SESSION_NEW_CONNECTION_EVENT;
-	event.reason = XIO_E_SUCCESS;
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_NEW_CONNECTION_EVENT,
+		.reason = XIO_E_SUCCESS,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -339,12 +345,14 @@ void xio_session_notify_connection_established(
 		struct xio_session *session,
 		struct xio_connection *connection)
 {
-	struct xio_session_event_data  event = {};
-
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
-	event.event = XIO_SESSION_CONNECTION_ESTABLISHED_EVENT;
-	event.reason = XIO_E_SUCCESS;
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_ESTABLISHED_EVENT,
+		.reason = XIO_E_SUCCESS,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -365,16 +373,19 @@ void xio_session_notify_connection_established(
 void xio_session_notify_connection_closed(struct xio_session *session,
 					  struct xio_connection *connection)
 {
-	struct xio_session_event_data  event = {};
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_CLOSED_EVENT,
+		.reason = (enum xio_status)connection->close_reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (connection->cd_bit)
 		return;
 
 	connection->cd_bit = 1;
-	event.event = XIO_SESSION_CONNECTION_CLOSED_EVENT;
-	event.reason = (enum xio_status)connection->close_reason;
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -397,18 +408,20 @@ void xio_session_notify_connection_disconnected(
 		struct xio_connection *connection,
 		enum xio_status reason)
 {
-	struct xio_session_event_data  event = {};
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_DISCONNECTED_EVENT,
+		.private_data = NULL,
+		.private_data_len = 0,
+		.reason = reason,
+	};
 
 	if (connection->cd_bit)
 		return;
 
 	connection->cd_bit = 1;
 
-	event.event = XIO_SESSION_CONNECTION_DISCONNECTED_EVENT;
-	event.reason = reason;
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
-
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
 		xio_ctx_debug_thread_unlock(connection->ctx);
@@ -429,12 +442,14 @@ void xio_session_notify_connection_refused(struct xio_session *session,
 					   struct xio_connection *connection,
 					   enum xio_status reason)
 {
-	struct xio_session_event_data  event = {};
-
-	event.event = XIO_SESSION_CONNECTION_REFUSED_EVENT;
-	event.reason = reason;
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_REFUSED_EVENT,
+		.reason = reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -455,12 +470,14 @@ void xio_session_notify_connection_refused(struct xio_session *session,
 void xio_session_notify_connection_teardown(struct xio_session *session,
 					    struct xio_connection *connection)
 {
-	struct xio_session_event_data  event = {};
-
-	event.event = XIO_SESSION_CONNECTION_TEARDOWN_EVENT;
-	event.reason = (enum xio_status)connection->close_reason;
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_TEARDOWN_EVENT,
+		.reason = (enum xio_status)connection->close_reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -482,12 +499,70 @@ void xio_session_notify_connection_error(struct xio_session *session,
 					 struct xio_connection *connection,
 					 enum xio_status reason)
 {
-	struct xio_session_event_data  event = {};
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_ERROR_EVENT,
+		.reason = reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
-	event.event = XIO_SESSION_CONNECTION_ERROR_EVENT;
-	event.reason = reason;
-	event.conn = connection;
-	event.conn_user_context = connection->cb_user_context;
+	if (session->ses_ops.on_session_event) {
+#ifdef XIO_THREAD_SAFE_DEBUG
+		xio_ctx_debug_thread_unlock(connection->ctx);
+#endif
+		session->ses_ops.on_session_event(
+				session, &event,
+				session->cb_user_context);
+#ifdef XIO_THREAD_SAFE_DEBUG
+		xio_ctx_debug_thread_lock(connection->ctx);
+#endif
+	}
+}
+
+/*---------------------------------------------------------------------------*/
+/* xio_session_notify_reconnecting										     */
+/*---------------------------------------------------------------------------*/
+void xio_session_notify_reconnecting(struct xio_session *session,
+		  struct xio_connection *connection)
+{
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_RECONNECTING_EVENT,
+		.reason = (enum xio_status)connection->close_reason,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
+
+	if (session->ses_ops.on_session_event) {
+#ifdef XIO_THREAD_SAFE_DEBUG
+		xio_ctx_debug_thread_unlock(connection->ctx);
+#endif
+		session->ses_ops.on_session_event(
+				session, &event,
+				session->cb_user_context);
+#ifdef XIO_THREAD_SAFE_DEBUG
+		xio_ctx_debug_thread_lock(connection->ctx);
+#endif
+	}
+}
+
+/*---------------------------------------------------------------------------*/
+/* xio_session_notify_reconnected										     */
+/*---------------------------------------------------------------------------*/
+void xio_session_notify_reconnected(struct xio_session *session,
+		  struct xio_connection *connection)
+{
+	struct xio_session_event_data  event = {
+		.conn = connection,
+		.conn_user_context = connection->cb_user_context,
+		.event = XIO_SESSION_CONNECTION_RECONNECTED_EVENT,
+		.reason = XIO_E_SUCCESS,
+		.private_data = NULL,
+		.private_data_len = 0,
+	};
 
 	if (session->ses_ops.on_session_event) {
 #ifdef XIO_THREAD_SAFE_DEBUG
@@ -1070,6 +1145,26 @@ int xio_on_nexus_disconnected(struct xio_session *session,
 	return 0;
 }
 
+/*---------------------------------------------------------------------------*/
+/* xio_on_nexus_reconnecting		                             */
+/*---------------------------------------------------------------------------*/
+int xio_on_nexus_reconnecting(struct xio_session *session,
+			     struct xio_nexus *nexus)
+{
+	struct xio_connection		*connection;
+
+	if (session->lead_connection &&
+	    session->lead_connection->nexus == nexus)
+		connection = session->lead_connection;
+	else
+		connection = xio_session_find_connection(session, nexus);
+
+	if (connection)
+		xio_connection_reconnect(connection);
+
+	return 0;
+}
+
 /*---------------------------------------------------------------------------*/
 /* xio_on_nexus_reconnected			                             */
 /*---------------------------------------------------------------------------*/
@@ -1774,6 +1869,10 @@ const char *xio_session_event_str(enum xio_session_event event)
 		return "connection error";
 	case XIO_SESSION_ERROR_EVENT:
 		return "session error";
+	case XIO_SESSION_CONNECTION_RECONNECTING_EVENT:
+		return "connection reconnecting";
+	case XIO_SESSION_CONNECTION_RECONNECTED_EVENT:
+		return "connection reconnected";
 	};
 	return "unknown session event";
 }
