@@ -146,6 +146,7 @@ struct xio_context *xio_context_create(struct xio_context_params *ctx_params,
 				max(ctx_params->max_conns_per_ctx, 2);
                 ctx->register_internal_mempool =
                         !!ctx_params->register_internal_mempool;
+		ctx->rq_depth = ctx_params->rq_depth;
 	}
 	if (!ctx->max_conns_per_ctx)
 		ctx->max_conns_per_ctx = 100;
@@ -500,7 +501,7 @@ int xio_context_run_loop(struct xio_context *ctx, int timeout_ms)
 #endif
 
 	ctx->is_running = 1;
-	retval = (timeout_ms == -1) ? xio_ev_loop_run(ctx->ev_loop) :
+	retval = (timeout_ms == XIO_INFINITE) ? xio_ev_loop_run(ctx->ev_loop) :
 		  xio_ev_loop_run_timeout(ctx->ev_loop, timeout_ms);
 	ctx->is_running = 0;
 
