@@ -109,12 +109,15 @@ struct test_params {
 /* globals								     */
 /*---------------------------------------------------------------------------*/
 static struct xio_test_config  test_config = {
-	XIO_DEF_ADDRESS,
-	XIO_DEF_PORT,
-	XIO_DEF_TRANSPORT,
-	XIO_DEF_CPU,
-	XIO_DEF_HEADER_SIZE,
-	XIO_DEF_DATA_SIZE
+	.server_addr = XIO_DEF_ADDRESS,
+	.server_port = XIO_DEF_PORT,
+	.transport = XIO_DEF_TRANSPORT,
+	.cpu = XIO_DEF_CPU,
+	.hdr_len = XIO_DEF_HEADER_SIZE,
+	.data_len = XIO_DEF_DATA_SIZE,
+	.conn_idx = 0,
+	.finite_run = 0,
+	.padding = { 0 },
 };
 
 /*---------------------------------------------------------------------------*/
